from bospy import bos

def ValueByNameTest(name:str|list[str]):
    pts = bos.NameToPoint(name)
    values = bos.Get(pts)
    print(name, "->", values)

def SetByQueryTest(t:str, l:str):
    pts = bos.QueryPoints(t, l)
    results = bos.Set(pts, "true")
    print(results)

if __name__ == "__main__":

    name1 = "air_temp" # a name that returns 1 point
    ValueByNameTest(name1)

    name2 = "status"
    ValueByNameTest([name1, name2])

    loc1 = "home"
    type1 = "brick:On_Command"
    SetByQueryTest(t=type1, l=loc1)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from . import comms_pb2 as comms__pb2

GRPC_GENERATED_VERSION = '1.66.0'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in comms_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class GetSetRunStub(object):
    """the GetSetRun service provides the fundamental driver functionality for Setting 
    values, getting them, and running commands.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Get = channel.unary_unary(
                '/bos.GetSetRun/Get',
                request_serializer=comms__pb2.GetRequest.SerializeToString,
                response_deserializer=comms__pb2.GetResponse.FromString,
                _registered_method=True)
        self.Set = channel.unary_unary(
                '/bos.GetSetRun/Set',
                request_serializer=comms__pb2.SetRequest.SerializeToString,
                response_deserializer=comms__pb2.SetResponse.FromString,
                _registered_method=True)


class GetSetRunServicer(object):
    """the GetSetRun service provides the fundamental driver functionality for Setting 
    values, getting them, and running commands.
    """

    def Get(self, request, context):
        """rpc for getting a value from a driver
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Set(self, request, context):
        """rpc for setting a value on a driver
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_GetSetRunServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Get': grpc.unary_unary_rpc_method_handler(
                    servicer.Get,
                    request_deserializer=comms__pb2.GetRequest.FromString,
                    response_serializer=comms__pb2.GetResponse.SerializeToString,
            ),
            'Set': grpc.unary_unary_rpc_method_handler(
                    servicer.Set,
                    request_deserializer=comms__pb2.SetRequest.FromString,
                    response_serializer=comms__pb2.SetResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'bos.GetSetRun', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('bos.GetSetRun', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class GetSetRun(object):
    """the GetSetRun service provides the fundamental driver functionality for Setting 
    values, getting them, and running commands.
    """

    @staticmethod
    def Get(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/bos.GetSetRun/Get',
            comms__pb2.GetRequest.SerializeToString,
            comms__pb2.GetResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def Set(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/bos.GetSetRun/Set',
            comms__pb2.SetRequest.SerializeToString,
            comms__pb2.SetResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)


class SysmodStub(object):
    """the PointId (pid) service takes classes, names, or regexes
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.NameToPoint = channel.unary_unary(
                '/bos.Sysmod/NameToPoint',
                request_serializer=comms__pb2.GetRequest.SerializeToString,
                response_deserializer=comms__pb2.QueryResponse.FromString,
                _registered_method=True)
        self.PointToName = channel.unary_unary(
                '/bos.Sysmod/PointToName',
                request_serializer=comms__pb2.GetRequest.SerializeToString,
                response_deserializer=comms__pb2.QueryResponse.FromString,
                _registered_method=True)
        self.TypeToPoint = channel.unary_unary(
                '/bos.Sysmod/TypeToPoint',
                request_serializer=comms__pb2.GetRequest.SerializeToString,
                response_deserializer=comms__pb2.QueryResponse.FromString,
                _registered_method=True)
        self.LocationToPoint = channel.unary_unary(
                '/bos.Sysmod/LocationToPoint',
                request_serializer=comms__pb2.GetRequest.SerializeToString,
                response_deserializer=comms__pb2.QueryResponse.FromString,
                _registered_method=True)
        self.QueryPoints = channel.unary_unary(
                '/bos.Sysmod/QueryPoints',
                request_serializer=comms__pb2.PointQueryRequest.SerializeToString,
                response_deserializer=comms__pb2.QueryResponse.FromString,
                _registered_method=True)
        self.GetDriverXref = channel.unary_unary(
                '/bos.Sysmod/GetDriverXref',
                request_serializer=comms__pb2.GetRequest.SerializeToString,
                response_deserializer=comms__pb2.QueryResponse.FromString,
                _registered_method=True)


class SysmodServicer(object):
    """the PointId (pid) service takes classes, names, or regexes
    """

    def NameToPoint(self, request, context):
        """Key are the point names. Response is POINT_ID or POINT_ID_LIST
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PointToName(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TypeToPoint(self, request, context):
        """Key are the class. Response is POINT_ID or POINT_ID_LIST
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def LocationToPoint(self, request, context):
        """Key is the location. Response is POINT_ID or POINT_ID_LIST
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def QueryPoints(self, request, context):
        """Takes a set of query params. Response is POINT_ID or POINT_ID_LIST
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDriverXref(self, request, context):
        """Takes a bos:Point. Returns the bos:xref of the corresponding driver 
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_SysmodServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'NameToPoint': grpc.unary_unary_rpc_method_handler(
                    servicer.NameToPoint,
                    request_deserializer=comms__pb2.GetRequest.FromString,
                    response_serializer=comms__pb2.QueryResponse.SerializeToString,
            ),
            'PointToName': grpc.unary_unary_rpc_method_handler(
                    servicer.PointToName,
                    request_deserializer=comms__pb2.GetRequest.FromString,
                    response_serializer=comms__pb2.QueryResponse.SerializeToString,
            ),
            'TypeToPoint': grpc.unary_unary_rpc_method_handler(
                    servicer.TypeToPoint,
                    request_deserializer=comms__pb2.GetRequest.FromString,
                    response_serializer=comms__pb2.QueryResponse.SerializeToString,
            ),
            'LocationToPoint': grpc.unary_unary_rpc_method_handler(
                    servicer.LocationToPoint,
                    request_deserializer=comms__pb2.GetRequest.FromString,
                    response_serializer=comms__pb2.QueryResponse.SerializeToString,
            ),
            'QueryPoints': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryPoints,
                    request_deserializer=comms__pb2.PointQueryRequest.FromString,
                    response_serializer=comms__pb2.QueryResponse.SerializeToString,
            ),
            'GetDriverXref': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDriverXref,
                    request_deserializer=comms__pb2.GetRequest.FromString,
                    response_serializer=comms__pb2.QueryResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'bos.Sysmod', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('bos.Sysmod', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class Sysmod(object):
    """the PointId (pid) service takes classes, names, or regexes
    """

    @staticmethod
    def NameToPoint(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/bos.Sysmod/NameToPoint',
            comms__pb2.GetRequest.SerializeToString,
            comms__pb2.QueryResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def PointToName(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/bos.Sysmod/PointToName',
            comms__pb2.GetRequest.SerializeToString,
            comms__pb2.QueryResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def TypeToPoint(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/bos.Sysmod/TypeToPoint',
            comms__pb2.GetRequest.SerializeToString,
            comms__pb2.QueryResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def LocationToPoint(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/bos.Sysmod/LocationToPoint',
            comms__pb2.GetRequest.SerializeToString,
            comms__pb2.QueryResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def QueryPoints(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/bos.Sysmod/QueryPoints',
            comms__pb2.PointQueryRequest.SerializeToString,
            comms__pb2.QueryResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetDriverXref(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/bos.Sysmod/GetDriverXref',
            comms__pb2.GetRequest.SerializeToString,
            comms__pb2.QueryResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)


class HealthCheckStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Ping = channel.unary_unary(
                '/bos.HealthCheck/Ping',
                request_serializer=comms__pb2.Empty.SerializeToString,
                response_deserializer=comms__pb2.Empty.FromString,
                _registered_method=True)


class HealthCheckServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Ping(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_HealthCheckServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Ping': grpc.unary_unary_rpc_method_handler(
                    servicer.Ping,
                    request_deserializer=comms__pb2.Empty.FromString,
                    response_serializer=comms__pb2.Empty.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'bos.HealthCheck', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('bos.HealthCheck', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class HealthCheck(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Ping(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/bos.HealthCheck/Ping',
            comms__pb2.Empty.SerializeToString,
            comms__pb2.Empty.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



from bospy import bos

from typing import Any

def SetTest(bosPtUri:str, value:str):
    ok = bos.Set(bosPtUri, value)
    if ok:
        print(bosPtUri, "<-", value, "(ok)")
    else:
        print(bosPtUri, "<-", value, "(SetError)")

def SetMultipleTest(keys:list[str], values:list[Any]):
    R = bos.Set(keys, values, full_response=True)
    for r in R:
        bosPtUri = r.Key
        valueStr = r.ValueStr # drivers are not required to provide this in response
        if r.Ok:
            print(bosPtUri, "<-", valueStr, "(ok)")
        else:
            print(bosPtUri, "<-", valueStr, "(SetError)")

if __name__ == "__main__":
    # print("sysmod address: ", bos.SYSMOD_ADDR)
    # print("devctrl address:", bos.DEVCTRL_ADDR)
    
    pt1 = "bos://localhost/dev/1/pts/1"
    SetTest(pt1, 420)

    pt2 = "bos://localhost/dev/1/pts/2"
    pt3 = "bos://localhost/dev/1/pts/4"
    pt4 = "bos://localhost/dev/1/pts/5"
    SetMultipleTest(
        [pt2, pt3, pt4],
        [18, 80, True],
        )
    
    # if n pts and 1 value are passed, the value is assigned to all n pts
    SetMultipleTest(
         [pt2, pt3], 
         20
    )

    pt5 = "bos://localhost/dev/2/pts/2"
    SetTest(pt5, False)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: comms.proto
# Protobuf Python Version: 5.27.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    27,
    2,
    '',
    'comms.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0b\x63omms.proto\x12\x03\x62os\"\x07\n\x05\x45mpty\"0\n\x06Header\x12\x0b\n\x03Src\x18\x01 \x01(\t\x12\x0b\n\x03\x44st\x18\x02 \x01(\t\x12\x0c\n\x04TxId\x18\x03 \x01(\x04\"\xa0\x01\n\x07GetPair\x12\x0b\n\x03Key\x18\x01 \x01(\t\x12\r\n\x05Value\x18\x02 \x01(\t\x12\x1e\n\x05\x44type\x18\x03 \x01(\x0e\x32\n.bos.DtypeH\x00\x88\x01\x01\x12!\n\x05\x45rror\x18\x05 \x01(\x0e\x32\r.bos.GetErrorH\x01\x88\x01\x01\x12\x15\n\x08\x45rrorMsg\x18\x06 \x01(\tH\x02\x88\x01\x01\x42\x08\n\x06_DtypeB\x08\n\x06_ErrorB\x0b\n\t_ErrorMsg\"\xac\x01\n\x07SetPair\x12\x0b\n\x03Key\x18\x01 \x01(\t\x12\r\n\x05Value\x18\x02 \x01(\t\x12\x1e\n\x05\x44type\x18\x03 \x01(\x0e\x32\n.bos.DtypeH\x00\x88\x01\x01\x12\n\n\x02Ok\x18\x04 \x01(\x08\x12!\n\x05\x45rror\x18\x05 \x01(\x0e\x32\r.bos.SetErrorH\x01\x88\x01\x01\x12\x15\n\x08\x45rrorMsg\x18\x06 \x01(\tH\x02\x88\x01\x01\x42\x08\n\x06_DtypeB\x08\n\x06_ErrorB\x0b\n\t_ErrorMsg\"7\n\nGetRequest\x12\x1b\n\x06Header\x18\x01 \x01(\x0b\x32\x0b.bos.Header\x12\x0c\n\x04Keys\x18\x02 \x03(\t\"\xac\x01\n\x0bGetResponse\x12 \n\x06Header\x18\x01 \x01(\x0b\x32\x0b.bos.HeaderH\x00\x88\x01\x01\x12\x1b\n\x05Pairs\x18\x02 \x03(\x0b\x32\x0c.bos.GetPair\x12%\n\x05\x45rror\x18\x03 \x01(\x0e\x32\x11.bos.ServiceErrorH\x01\x88\x01\x01\x12\x15\n\x08\x45rrorMsg\x18\x04 \x01(\tH\x02\x88\x01\x01\x42\t\n\x07_HeaderB\x08\n\x06_ErrorB\x0b\n\t_ErrorMsg\"V\n\nSetRequest\x12 \n\x06Header\x18\x01 \x01(\x0b\x32\x0b.bos.HeaderH\x00\x88\x01\x01\x12\x1b\n\x05Pairs\x18\x02 \x03(\x0b\x32\x0c.bos.SetPairB\t\n\x07_Header\"\xac\x01\n\x0bSetResponse\x12 \n\x06Header\x18\x01 \x01(\x0b\x32\x0b.bos.HeaderH\x00\x88\x01\x01\x12\x1b\n\x05Pairs\x18\x02 \x03(\x0b\x32\x0c.bos.SetPair\x12%\n\x05\x45rror\x18\x03 \x01(\x0e\x32\x11.bos.ServiceErrorH\x01\x88\x01\x01\x12\x15\n\x08\x45rrorMsg\x18\x04 \x01(\tH\x02\x88\x01\x01\x42\t\n\x07_HeaderB\x08\n\x06_ErrorB\x0b\n\t_ErrorMsg\"\xe8\x02\n\x11PointQueryRequest\x12 \n\x06Header\x18\x01 \x01(\x0b\x32\x0b.bos.HeaderH\x00\x88\x01\x01\x12\x12\n\x05Query\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x13\n\x06\x44\x65vice\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\r\n\x05Names\x18\x04 \x03(\t\x12\r\n\x05Types\x18\x05 \x03(\t\x12\x11\n\tLocations\x18\x06 \x03(\t\x12\x1e\n\x11\x43onsiderDeviceLoc\x18\x07 \x01(\x08H\x03\x88\x01\x01\x12!\n\x08Resource\x18\x08 \x01(\x0e\x32\n.bos.DtypeH\x04\x88\x01\x01\x12#\n\x05\x45rror\x18\x0b \x01(\x0e\x32\x0f.bos.QueryErrorH\x05\x88\x01\x01\x12\x15\n\x08\x45rrorMsg\x18\x0c \x01(\tH\x06\x88\x01\x01\x42\t\n\x07_HeaderB\x08\n\x06_QueryB\t\n\x07_DeviceB\x14\n\x12_ConsiderDeviceLocB\x0b\n\t_ResourceB\x08\n\x06_ErrorB\x0b\n\t_ErrorMsg\"\xe7\x01\n\rQueryResponse\x12 \n\x06Header\x18\x01 \x01(\x0b\x32\x0b.bos.HeaderH\x00\x88\x01\x01\x12\x12\n\x05Query\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x0e\n\x06Values\x18\x03 \x03(\t\x12\x1e\n\x05\x44type\x18\x04 \x01(\x0e\x32\n.bos.DtypeH\x02\x88\x01\x01\x12#\n\x05\x45rror\x18\x05 \x01(\x0e\x32\x0f.bos.QueryErrorH\x03\x88\x01\x01\x12\x15\n\x08\x45rrorMsg\x18\x06 \x01(\tH\x04\x88\x01\x01\x42\t\n\x07_HeaderB\x08\n\x06_QueryB\x08\n\x06_DtypeB\x08\n\x06_ErrorB\x0b\n\t_ErrorMsg*\xa0\x01\n\x0cServiceError\x12\x16\n\x12SERVICE_ERROR_NONE\x10\x00\x12\x1d\n\x19SERVICE_ERROR_UNSPECIFIED\x10\x01\x12\x1d\n\x19SERVICE_ERROR_NO_RESPONSE\x10\x02\x12\x19\n\x15SERVICE_ERROR_TIMEOUT\x10\x03\x12\x1f\n\x1bSERVICE_ERROR_ACCESS_DENIED\x10\x07*\x83\x02\n\x08GetError\x12\x12\n\x0eGET_ERROR_NONE\x10\x00\x12\x19\n\x15GET_ERROR_UNSPECIFIED\x10\x01\x12 \n\x1cGET_ERROR_KEY_DOES_NOT_EXIST\x10\x02\x12\x15\n\x11GET_ERROR_TIMEOUT\x10\x03\x12&\n\"GET_ERROR_COULD_NOT_RESOLVE_DRIVER\x10\x04\x12$\n GET_ERROR_COULD_NOT_RESOLVE_ADDR\x10\x05\x12$\n GET_ERROR_COULD_NOT_RESOLVE_XREF\x10\x06\x12\x1b\n\x17GET_ERROR_ACCESS_DENIED\x10\x07*\xbe\x02\n\x08SetError\x12\x12\n\x0eSET_ERROR_NONE\x10\x00\x12\x19\n\x15SET_ERROR_UNSPECIFIED\x10\x01\x12 \n\x1cSET_ERROR_KEY_DOES_NOT_EXIST\x10\x02\x12\x15\n\x11SET_ERROR_TIMEOUT\x10\x03\x12&\n\"SET_ERROR_COULD_NOT_RESOLVE_DRIVER\x10\x04\x12$\n SET_ERROR_COULD_NOT_RESOLVE_ADDR\x10\x05\x12$\n SET_ERROR_COULD_NOT_RESOLVE_XREF\x10\x06\x12\x1b\n\x17SET_ERROR_ACCESS_DENIED\x10\x07\x12\x17\n\x13SET_ERROR_READ_ONLY\x10\x08\x12 \n\x1cSET_ERROR_INVALID_VALUE_TYPE\x10\t*\x97\x01\n\nQueryError\x12\x14\n\x10QUERY_ERROR_NONE\x10\x00\x12\x1b\n\x17QUERY_ERROR_UNSPECIFIED\x10\x01\x12\x17\n\x13QUERY_ERROR_TIMEOUT\x10\x03\x12\x1e\n\x1aQUERY_ERROR_UNKNOWN_PREFIX\x10\x04\x12\x1d\n\x19QUERY_ERROR_ACCESS_DENIED\x10\x05*\xab\x02\n\x05\x44type\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x08\n\x04NULL\x10\x01\x12\n\n\x06\x44OUBLE\x10\n\x12\t\n\x05\x46LOAT\x10\x0b\x12\t\n\x05INT32\x10\x0c\x12\t\n\x05INT64\x10\r\x12\n\n\x06UINT32\x10\x0e\x12\n\n\x06UINT64\x10\x0f\x12\n\n\x06SINT32\x10\x10\x12\n\n\x06SINT64\x10\x11\x12\x0b\n\x07\x46IXED32\x10\x12\x12\x0b\n\x07\x46IXED64\x10\x13\x12\x0c\n\x08SFIXED32\x10\x14\x12\x0c\n\x08SFIXED64\x10\x15\x12\x08\n\x04\x42OOL\x10\x16\x12\n\n\x06STRING\x10\x17\x12\t\n\x05\x42YTES\x10\x18\x12\t\n\x05POINT\x10\x1e\x12\x0e\n\nPOINT_LIST\x10\x1f\x12\n\n\x06\x44\x45VICE\x10(\x12\x0f\n\x0b\x44\x45VICE_LIST\x10)\x12\n\n\x06\x44RIVER\x10\x30\x12\x0f\n\x0b\x44RIVER_XREF\x10\x31\x32_\n\tGetSetRun\x12(\n\x03Get\x12\x0f.bos.GetRequest\x1a\x10.bos.GetResponse\x12(\n\x03Set\x12\x0f.bos.SetRequest\x1a\x10.bos.SetResponse2\xcd\x02\n\x06Sysmod\x12\x32\n\x0bNameToPoint\x12\x0f.bos.GetRequest\x1a\x12.bos.QueryResponse\x12\x32\n\x0bPointToName\x12\x0f.bos.GetRequest\x1a\x12.bos.QueryResponse\x12\x32\n\x0bTypeToPoint\x12\x0f.bos.GetRequest\x1a\x12.bos.QueryResponse\x12\x36\n\x0fLocationToPoint\x12\x0f.bos.GetRequest\x1a\x12.bos.QueryResponse\x12\x39\n\x0bQueryPoints\x12\x16.bos.PointQueryRequest\x1a\x12.bos.QueryResponse\x12\x34\n\rGetDriverXref\x12\x0f.bos.GetRequest\x1a\x12.bos.QueryResponse2-\n\x0bHealthCheck\x12\x1e\n\x04Ping\x12\n.bos.Empty\x1a\n.bos.Emptyb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'comms_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_SERVICEERROR']._serialized_start=1510
  _globals['_SERVICEERROR']._serialized_end=1670
  _globals['_GETERROR']._serialized_start=1673
  _globals['_GETERROR']._serialized_end=1932
  _globals['_SETERROR']._serialized_start=1935
  _globals['_SETERROR']._serialized_end=2253
  _globals['_QUERYERROR']._serialized_start=2256
  _globals['_QUERYERROR']._serialized_end=2407
  _globals['_DTYPE']._serialized_start=2410
  _globals['_DTYPE']._serialized_end=2709
  _globals['_EMPTY']._serialized_start=20
  _globals['_EMPTY']._serialized_end=27
  _globals['_HEADER']._serialized_start=29
  _globals['_HEADER']._serialized_end=77
  _globals['_GETPAIR']._serialized_start=80
  _globals['_GETPAIR']._serialized_end=240
  _globals['_SETPAIR']._serialized_start=243
  _globals['_SETPAIR']._serialized_end=415
  _globals['_GETREQUEST']._serialized_start=417
  _globals['_GETREQUEST']._serialized_end=472
  _globals['_GETRESPONSE']._serialized_start=475
  _globals['_GETRESPONSE']._serialized_end=647
  _globals['_SETREQUEST']._serialized_start=649
  _globals['_SETREQUEST']._serialized_end=735
  _globals['_SETRESPONSE']._serialized_start=738
  _globals['_SETRESPONSE']._serialized_end=910
  _globals['_POINTQUERYREQUEST']._serialized_start=913
  _globals['_POINTQUERYREQUEST']._serialized_end=1273
  _globals['_QUERYRESPONSE']._serialized_start=1276
  _globals['_QUERYRESPONSE']._serialized_end=1507
  _globals['_GETSETRUN']._serialized_start=2711
  _globals['_GETSETRUN']._serialized_end=2806
  _globals['_SYSMOD']._serialized_start=2809
  _globals['_SYSMOD']._serialized_end=3142
  _globals['_HEALTHCHECK']._serialized_start=3144
  _globals['_HEALTHCHECK']._serialized_end=3189
# @@protoc_insertion_point(module_scope)



from bospy import bos, utils

simple_output = True

def NameTest(name:str):
    pt = bos.NameToPoint(name)
    if simple_output:
        pt = utils.SimplifyPoint(pt)
    print("== point(s) named {} ==".format(name))
    print("\t", pt)
    return pt
    
def NameTest2(name:str):
    pts = bos.NameToPoint(name, multiple_matches=True)
    if simple_output:
        pts = utils.SimplifyPoint(pts)
    print("== point(s) named {} ==".format(name))
    print("\t", pts)
    return pts

def PointNameTest(pt:str):
    name = bos.PointToName(pt)
    if simple_output:
        pt = utils.SimplifyPoint(pt)
    print("== Name of {} ==".format(pt))
    print("\t", name)
    return name

def TypeTest(_type:str):
    pts = bos.TypeToPoint(_type)
    if simple_output:
        pts = utils.SimplifyPoint(pts)
        _type = utils.SimplifyBrickType(_type)
    print("== point(s) typed {} ==".format(_type))
    print("\t", pts)
    return pts

def LocationTest(location:str):
    pts = bos.LocationToPoint(location)
    if simple_output:
        pts = utils.SimplifyPoint(pts)
    print("== point(s) located in '{}' ==".format(location))
    print("\t", " ".join(pts))
    return pts

def QueryAllTest():
    bos.QueryPoints()

def QueryTest(types:str|list[str]=None, locations:str|list[str]=None):
    if type(locations) == str:
        locations = [locations]
    if type(types) == str:
        types == [types]
    pts = bos.QueryPoints(types, locations)
    if simple_output:
        pts = [utils.SimplifyPoint(pt) for pt in pts]
        types = [utils.SimplifyBrickType(types) for t in types]
    print("== point(s) located in '{}' with type {} ==".format(locations[0], types[0]))
    print("\t", " ".join(pts))
    return pts

if __name__ == "__main__":
    name1 = "air_temp" # will only return 1 point
    name2 = "voltage"     # will return more than 1 point

    NameTest(name1)
    NameTest2(name2)

    ptUri1 = "bos://localhost/dev/1/pts/3"
    PointNameTest(ptUri1)

    type1 = "https://brickschema.org/schema/Brick#Air_Temperature_Sensor"
    TypeTest(type1)

    location1 = "home"
    LocationTest(location1)

    # location2 = "lab"
    QueryTest(type1, location1)


from bospy import bos, utils

def GetTest(bosPtUri:str):
    values = bos.Get(bosPtUri)
    for k, v in values.items():
        print(k, "->", v, "({})".format(type(v)))
    
def GetMultipleTest(bosPtUris:list[str]):
    R = bos.Get(bosPtUris)
    for key, value in R.items():
        print(key, "->", value, "({})".format(type(value)))

if __name__ == "__main__":
    print("sysmod address: ", bos.SYSMOD_ADDR)
    print("devctrl address:", bos.SYSMOD_ADDR)
    
    test1_url = "bos://localhost/dev/1/pts/1"
    GetTest(test1_url)

    test2_url = "bos://localhost/dev/1/pts/2"
    test3_url = "bos://localhost/dev/1/pts/3"
    test4_url = "bos://localhost/dev/1/pts/4"

    GetTest([test2_url, test3_url, test4_url])
    


from . import comms_pb2_grpc
from . import comms_pb2
import grpc

import datetime as dt
import sys
import os

from typing import Any

""" Provides the wrapper functions used to access openBOS points in Python
"""

VERSION = "0.0.4"

SYSMOD_ADDR = os.environ.get('SYSMOD_ADDR')
DEVCTRL_ADDR = os.environ.get('DEVCTRL_ADDR')

# uri -> name cache
point_name_cache = {}

# apply defaults
if SYSMOD_ADDR is None:
    SYSMOD_ADDR = "localhost:2821"
if DEVCTRL_ADDR is None:
    DEVCTRL_ADDR = "localhost:2822"

# client calls for the sysmod rpc calls
def NameToPoint(names:str|list[str], multiple_matches:bool=False, addr:str=SYSMOD_ADDR) -> None | list[str]:
    if isinstance(names, str):
        names = [names]
    else:
        multiple_matches = True

    response: comms_pb2.QueryResponse
    with grpc.insecure_channel(addr) as channel:
        stub = comms_pb2_grpc.SysmodStub(channel)
        response = stub.NameToPoint(comms_pb2.GetRequest(
            Keys=names
        ))
        if response.Error > 0:
            print("get '{}' error: {}".format(response.Query,
                                              response.Error))
    # cast as a more user-friendly type
    if multiple_matches:
        return response.Values
    elif len(response.Values) == 1:
        return response.Values[0]
    else:
        return None
    
def PointToName(pt:str, addr:str=SYSMOD_ADDR) -> None | str:
    response: comms_pb2.QueryResponse
    with grpc.insecure_channel(addr) as channel:
        stub = comms_pb2_grpc.SysmodStub(channel)
        response = stub.PointToName(comms_pb2.GetRequest(
            Keys=[pt]
        ))
        if response.Error > 0:
            print("get '{}' error: {}".format(response.Query,
                                              response.Error))
    if len(response.Values) > 0:
        return response.Values[0]
    else:
        return None

def TypeToPoint(types:str|list[str], addr:str=SYSMOD_ADDR) -> None | str | list[str]:
    if isinstance(types, str):
        types = [types]
    response: comms_pb2.QueryResponse
    with grpc.insecure_channel(addr) as channel:
        stub = comms_pb2_grpc.SysmodStub(channel)
        response = stub.TypeToPoint(comms_pb2.GetRequest(
            Keys=types))
        if response.Error > 0:
            print("get '{}' error: {}".format(response.Query,
                                              response.Error))
    # cast as a more user-friendly type
    return response.Values

def LocationToPoint(locations:str|list[str], addr:str=SYSMOD_ADDR) -> None | str | list[str]:
    print(locations, type(locations))
    if isinstance(locations, str):
        locations = [locations]
    response: comms_pb2.QueryResponse
    with grpc.insecure_channel(addr) as channel:
        stub = comms_pb2_grpc.SysmodStub(channel)
        response = stub.LocationToPoint(comms_pb2.GetRequest(
            Keys=locations))
        if response.Error > 0:
            print("get '{}' error: {}".format(response.Query,
                                              response.Error))
    return response.Values

def QueryPoints(query:str=None, types:str|list[str]=None, locations:str|list[str]=None, inherit_device_loc:bool=True, addr:str=SYSMOD_ADDR):
    """ if query, types, and locations are all none. This returns all pts in sysmod.
    """
    print('The sysmod address is: {}'.format(addr))
    if isinstance(types, str):
        types = [types]
    if isinstance(locations, str):
        locations = [locations]

    response: comms_pb2.QueryResponse
    with grpc.insecure_channel(addr) as channel:
        stub = comms_pb2_grpc.SysmodStub(channel)
        if query is None:
            response = stub.QueryPoints(comms_pb2.PointQueryRequest(
                Types=types,
                Locations=locations,
                ConsiderDeviceLoc=inherit_device_loc,
            ))
        else:
            response = stub.QueryPoints(comms_pb2.PointQueryRequest(
                Query=query,
            ))
        if response.Error > 0:
            print("get '{}' error: {}".format(response.Query,
                                              response.Error))
    return response.Values

# devctrl rpc calls
class GetValue(object):
    def __init__(self):
        self.Key:str
        self.Value:str

class GetResponse(object):
    def __init__(self):
        self.Values:list[GetValue] = []


def NewGetValues(resp:comms_pb2.GetResponse) -> list[GetValue]:
    V:list[GetValue] = []
    for v in resp.Pairs:
        _v = GetValue()
        _v.Key = v.Key
        _v.Value = GetTypedValue(v)
        V.append(_v)
        
    return V


class SetResponse(object):
    def __init__(self):
        self.Key:str = None
        self.ValueStr:str = None
        self.Ok:bool = False


def NewSetResponse(responses:comms_pb2.SetResponse) -> list[SetResponse]:
    R:list[SetResponse] = []
    for p in responses.Pairs:
        r = SetResponse()
        if p.Key is not None:
            r.Key = p.Key
        if p.Value is not None:
            r.ValueStr = p.Value
        r.Ok = p.Ok
        R.append(r)
    return R


def Ping(addr:str) -> bool:
    response: comms_pb2.Empty
    with grpc.insecure_channel(addr) as channel:
        stub = comms_pb2_grpc.HealthCheckStub(channel)
        response = stub.Ping(comms_pb2.Empty())
    if response is not None:
        return True
    else:
        return False


def CheckLatency(addr:str, num_pings:int=5) -> dt.timedelta | None:
    running_total:dt.timedelta
    for i in range(num_pings):
        start = dt.datetime.now()
        ok = Ping(addr)
        end = dt.datetime.now()
        if not ok:
            return None
        diff = end-start
        if i == 0:
            running_total = diff
        else:
            running_total = running_total + diff
    return running_total / num_pings
        

def Get(keys:str|list[str], full_response=False, addr=DEVCTRL_ADDR) -> list[GetResponse] | dict[str, object]:
    if type(keys) == str:
        keys = [keys]

    response: comms_pb2.GetResponse
    with grpc.insecure_channel(addr) as channel:
        stub = comms_pb2_grpc.GetSetRunStub(channel)
        response = stub.Get(comms_pb2.GetRequest(Keys=keys))
    R = NewGetValues(response)
    if full_response:
        return R
    D = {}
    for r in R:
        D[r.Key] = r.Value
    return D

def Set(keys:str|list[str], values:str|list[str], full_response=False, addr=DEVCTRL_ADDR) -> SetResponse | dict[str, bool] | bool:
    if isinstance(keys, str):
        keys = [keys]
    if isinstance(values, (str, float, int, bool)):
        values = [values]

    # validate the number of keys and values
    if len(keys) != len(values) :
        if len(keys) >= 1 and len(values) == 1:
            values = [values[0]] * len(keys)
        else:
            print("error: unable to broadcast values to match number of keys")
            print("\thave {} keys and {} values".format(len(keys), len(values)))
            return False

    # by now now we must have an equal number of keys and values, format them
    pairs = [comms_pb2.SetPair(Key=k, Value=str(values[i])) for i, k in enumerate(keys)]

    response: comms_pb2.SetResponse
    with grpc.insecure_channel(addr) as channel:
        stub = comms_pb2_grpc.GetSetRunStub(channel)
        response = stub.Set(comms_pb2.SetRequest(Pairs=pairs))
        if response.Error > 0:
            print("SET_ERROR_{}: {}".format(response.Error, response.ErrorMsg))
            return False
    r = NewSetResponse(response)
    if full_response:
        return r
    return True


def GetTypedValue(v:comms_pb2.GetPair|comms_pb2.SetPair):
    """ a helper function that uses the appropriate fields from a comms_pb2.GetReponse
    to return a typed value.
    """
    return DecodeValue(v.Value, v.Dtype)


def DecodeValue(s:str, dtype:comms_pb2.Dtype=comms_pb2.UNSPECIFIED):
    if (dtype == comms_pb2.DOUBLE) or (dtype == comms_pb2.FLOAT):
        return float(s)
    if (dtype == comms_pb2.INT32) or (dtype == comms_pb2.INT64) or (dtype == comms_pb2.UINT32) or (dtype == comms_pb2.UINT64):
        return int(s)
    if (dtype == comms_pb2.BOOL):
        return bool(s)
    if (dtype == comms_pb2.STRING):
        return s
    else:
        return UntypedString(s)
    

class UntypedString(str):
    """ Used to show that a value received by Get or GetMultiple was cast to a 
    native python type but that the function did not receive dtype information 
    (i.e., the Dtype=UNSPECIFIED)
    """

class PointUri(str):
    """ Used to indicate that a value is not just a str but specifically a point uri.
    """


if __name__ == "__main__":
    """ running this file will do a health check on the devctrl and sysmod services.
    """
    devctrl_addr = os.environ.get('DEVCTRL_ADDR')
    if devctrl_addr is None:
        print("environment variable DEVCTRL_ADDR not set. Try running:")
        print("\t$ source serivces/config-env")
        sys.exit(1)

    # make sure devCtrl is running
    try:
        resp = CheckLatency(devctrl_addr)
    except Exception as e:
        print("devctrl did not respond at {}\n\tis it running?".format(devctrl_addr))
        template = "An exception of type {0} occurred. Arguments:\n{1!r}"
        message = template.format(type(e).__name__, e.args)
        print(message)
        sys.exit(1)
    else:
        print("devctrl running. RTT = {:.2f} ms".format(resp.total_seconds()*1000))

    sysmod_addr = os.environ.get('SYSMOD_ADDR')
    if sysmod_addr is None:
        print("environment variable DEVCTRL_ADDR not set. Try running:")
        print("\t$ source serivces/config-env")
        sys.exit(1)

    # make sure devCtrl is running
    try:
        resp = CheckLatency(sysmod_addr)
    except Exception as e:
        print("devCtrl did not respond at {}\n\tis it running?".format(sysmod_addr))
        template = "An exception of type {0} occurred. Arguments:\n{1!r}"
        message = template.format(type(e).__name__, e.args)
        print(message)
        sys.exit(1)
    else:
        print("sysmod running. RTT = {:.2f} ms".format(resp.total_seconds()*1000))



import services.wrappers.python.bos as c
import comms_pb2

if __name__ == "__main__":
    tests = [
        ("true", comms_pb2.BOOL),
        ("false", comms_pb2.BOOL),
        ("-1", comms_pb2.INT32),
        ("1", comms_pb2.UINT32),
        ("1.0", comms_pb2.FLOAT),
        ("string", comms_pb2.STRING),
        ("unspecified value", comms_pb2.UNSPECIFIED),
    ]

    answers = [
        bool,
        bool, 
        int, 
        int, 
        float, 
        str,
        str,
    ]

    assert len(tests) == len(answers)
    for i, case in enumerate(tests):
        typed_val = c.DecodeValue(*case)
        print("{} a {} ({})".format(case[0], type(typed_val), case[1]))
        assert type(typed_val) == answers[i]


import re
import os

point_re = re.compile(r'(?P<prefix>^bos://[A-Za-z0-9.-]+/dev/)(?P<point>[0-9]+/pts/[0-9]+$)')
dev_point_re = re.compile(r'(?P<device>[0-9]+)/pts/(?P<point>[0-9]+)')

brick_type_re = re.compile(r'^(?P<prefix>https://brickschema.org/schema/Brick#)(?P<type>[A-Za-z0-9_]+)')

def SimplifyPoint(pts:list[str]) -> list[str]:
    if type(pts) == str:
        pts = [pts]
    for i, p in enumerate(pts):
        m = point_re.match(p)
        if m is not None:
            prefix = m.groupdict()['prefix']
            pt_str = pts[i][len(prefix):]
            m2 = dev_point_re.match(pt_str)
            parts = [m2.groupdict()['device'], m2.groupdict()['point']]
            pts[i] = ".".join(parts)
    if len(pts) == 1:
        return pts[0]
    return pts

def SimplifyBrickType(_type:str):
    m = brick_type_re.match(_type)
    if m is not None:
        prefix = m.groupdict()['prefix']
        type_str = _type[len(prefix):]
        return "brick:" + type_str
    return _type

if __name__ == "__main__":
    p1 = "bos://localhost/dev/1/pts/5"
    p2 = "bos://localhost/dev/1/pts/5"
    p3 = "https://google.com/dev/1/pts/5"
    for p in [p1, p2, p3]:
        m = point_re.match(p)
        if m is None:
            print("not a match")
        else:
            print(m.groupdict())

    print(SimplifyPoint(p1))
    print(SimplifyPoint([p1]))
    print(SimplifyPoint([p1, p2]))

    t1 = "https://brickschema.org/schema/Brick#Air_Temperature_Sensor"
    t2 = "https://rec.com/schema/rec#Control_Zone"
    print(SimplifyBrickType(t1))
    print(SimplifyBrickType(t2))


from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ServiceError(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    SERVICE_ERROR_NONE: _ClassVar[ServiceError]
    SERVICE_ERROR_UNSPECIFIED: _ClassVar[ServiceError]
    SERVICE_ERROR_NO_RESPONSE: _ClassVar[ServiceError]
    SERVICE_ERROR_TIMEOUT: _ClassVar[ServiceError]
    SERVICE_ERROR_ACCESS_DENIED: _ClassVar[ServiceError]

class GetError(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    GET_ERROR_NONE: _ClassVar[GetError]
    GET_ERROR_UNSPECIFIED: _ClassVar[GetError]
    GET_ERROR_KEY_DOES_NOT_EXIST: _ClassVar[GetError]
    GET_ERROR_TIMEOUT: _ClassVar[GetError]
    GET_ERROR_COULD_NOT_RESOLVE_DRIVER: _ClassVar[GetError]
    GET_ERROR_COULD_NOT_RESOLVE_ADDR: _ClassVar[GetError]
    GET_ERROR_COULD_NOT_RESOLVE_XREF: _ClassVar[GetError]
    GET_ERROR_ACCESS_DENIED: _ClassVar[GetError]

class SetError(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    SET_ERROR_NONE: _ClassVar[SetError]
    SET_ERROR_UNSPECIFIED: _ClassVar[SetError]
    SET_ERROR_KEY_DOES_NOT_EXIST: _ClassVar[SetError]
    SET_ERROR_TIMEOUT: _ClassVar[SetError]
    SET_ERROR_COULD_NOT_RESOLVE_DRIVER: _ClassVar[SetError]
    SET_ERROR_COULD_NOT_RESOLVE_ADDR: _ClassVar[SetError]
    SET_ERROR_COULD_NOT_RESOLVE_XREF: _ClassVar[SetError]
    SET_ERROR_ACCESS_DENIED: _ClassVar[SetError]
    SET_ERROR_READ_ONLY: _ClassVar[SetError]
    SET_ERROR_INVALID_VALUE_TYPE: _ClassVar[SetError]

class QueryError(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    QUERY_ERROR_NONE: _ClassVar[QueryError]
    QUERY_ERROR_UNSPECIFIED: _ClassVar[QueryError]
    QUERY_ERROR_TIMEOUT: _ClassVar[QueryError]
    QUERY_ERROR_UNKNOWN_PREFIX: _ClassVar[QueryError]
    QUERY_ERROR_ACCESS_DENIED: _ClassVar[QueryError]

class Dtype(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    UNSPECIFIED: _ClassVar[Dtype]
    NULL: _ClassVar[Dtype]
    DOUBLE: _ClassVar[Dtype]
    FLOAT: _ClassVar[Dtype]
    INT32: _ClassVar[Dtype]
    INT64: _ClassVar[Dtype]
    UINT32: _ClassVar[Dtype]
    UINT64: _ClassVar[Dtype]
    SINT32: _ClassVar[Dtype]
    SINT64: _ClassVar[Dtype]
    FIXED32: _ClassVar[Dtype]
    FIXED64: _ClassVar[Dtype]
    SFIXED32: _ClassVar[Dtype]
    SFIXED64: _ClassVar[Dtype]
    BOOL: _ClassVar[Dtype]
    STRING: _ClassVar[Dtype]
    BYTES: _ClassVar[Dtype]
    POINT: _ClassVar[Dtype]
    POINT_LIST: _ClassVar[Dtype]
    DEVICE: _ClassVar[Dtype]
    DEVICE_LIST: _ClassVar[Dtype]
    DRIVER: _ClassVar[Dtype]
    DRIVER_XREF: _ClassVar[Dtype]
SERVICE_ERROR_NONE: ServiceError
SERVICE_ERROR_UNSPECIFIED: ServiceError
SERVICE_ERROR_NO_RESPONSE: ServiceError
SERVICE_ERROR_TIMEOUT: ServiceError
SERVICE_ERROR_ACCESS_DENIED: ServiceError
GET_ERROR_NONE: GetError
GET_ERROR_UNSPECIFIED: GetError
GET_ERROR_KEY_DOES_NOT_EXIST: GetError
GET_ERROR_TIMEOUT: GetError
GET_ERROR_COULD_NOT_RESOLVE_DRIVER: GetError
GET_ERROR_COULD_NOT_RESOLVE_ADDR: GetError
GET_ERROR_COULD_NOT_RESOLVE_XREF: GetError
GET_ERROR_ACCESS_DENIED: GetError
SET_ERROR_NONE: SetError
SET_ERROR_UNSPECIFIED: SetError
SET_ERROR_KEY_DOES_NOT_EXIST: SetError
SET_ERROR_TIMEOUT: SetError
SET_ERROR_COULD_NOT_RESOLVE_DRIVER: SetError
SET_ERROR_COULD_NOT_RESOLVE_ADDR: SetError
SET_ERROR_COULD_NOT_RESOLVE_XREF: SetError
SET_ERROR_ACCESS_DENIED: SetError
SET_ERROR_READ_ONLY: SetError
SET_ERROR_INVALID_VALUE_TYPE: SetError
QUERY_ERROR_NONE: QueryError
QUERY_ERROR_UNSPECIFIED: QueryError
QUERY_ERROR_TIMEOUT: QueryError
QUERY_ERROR_UNKNOWN_PREFIX: QueryError
QUERY_ERROR_ACCESS_DENIED: QueryError
UNSPECIFIED: Dtype
NULL: Dtype
DOUBLE: Dtype
FLOAT: Dtype
INT32: Dtype
INT64: Dtype
UINT32: Dtype
UINT64: Dtype
SINT32: Dtype
SINT64: Dtype
FIXED32: Dtype
FIXED64: Dtype
SFIXED32: Dtype
SFIXED64: Dtype
BOOL: Dtype
STRING: Dtype
BYTES: Dtype
POINT: Dtype
POINT_LIST: Dtype
DEVICE: Dtype
DEVICE_LIST: Dtype
DRIVER: Dtype
DRIVER_XREF: Dtype

class Empty(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class Header(_message.Message):
    __slots__ = ("Src", "Dst", "TxId")
    SRC_FIELD_NUMBER: _ClassVar[int]
    DST_FIELD_NUMBER: _ClassVar[int]
    TXID_FIELD_NUMBER: _ClassVar[int]
    Src: str
    Dst: str
    TxId: int
    def __init__(self, Src: _Optional[str] = ..., Dst: _Optional[str] = ..., TxId: _Optional[int] = ...) -> None: ...

class GetPair(_message.Message):
    __slots__ = ("Key", "Value", "Dtype", "Error", "ErrorMsg")
    KEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    DTYPE_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    ERRORMSG_FIELD_NUMBER: _ClassVar[int]
    Key: str
    Value: str
    Dtype: Dtype
    Error: GetError
    ErrorMsg: str
    def __init__(self, Key: _Optional[str] = ..., Value: _Optional[str] = ..., Dtype: _Optional[_Union[Dtype, str]] = ..., Error: _Optional[_Union[GetError, str]] = ..., ErrorMsg: _Optional[str] = ...) -> None: ...

class SetPair(_message.Message):
    __slots__ = ("Key", "Value", "Dtype", "Ok", "Error", "ErrorMsg")
    KEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    DTYPE_FIELD_NUMBER: _ClassVar[int]
    OK_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    ERRORMSG_FIELD_NUMBER: _ClassVar[int]
    Key: str
    Value: str
    Dtype: Dtype
    Ok: bool
    Error: SetError
    ErrorMsg: str
    def __init__(self, Key: _Optional[str] = ..., Value: _Optional[str] = ..., Dtype: _Optional[_Union[Dtype, str]] = ..., Ok: bool = ..., Error: _Optional[_Union[SetError, str]] = ..., ErrorMsg: _Optional[str] = ...) -> None: ...

class GetRequest(_message.Message):
    __slots__ = ("Header", "Keys")
    HEADER_FIELD_NUMBER: _ClassVar[int]
    KEYS_FIELD_NUMBER: _ClassVar[int]
    Header: Header
    Keys: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, Header: _Optional[_Union[Header, _Mapping]] = ..., Keys: _Optional[_Iterable[str]] = ...) -> None: ...

class GetResponse(_message.Message):
    __slots__ = ("Header", "Pairs", "Error", "ErrorMsg")
    HEADER_FIELD_NUMBER: _ClassVar[int]
    PAIRS_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    ERRORMSG_FIELD_NUMBER: _ClassVar[int]
    Header: Header
    Pairs: _containers.RepeatedCompositeFieldContainer[GetPair]
    Error: ServiceError
    ErrorMsg: str
    def __init__(self, Header: _Optional[_Union[Header, _Mapping]] = ..., Pairs: _Optional[_Iterable[_Union[GetPair, _Mapping]]] = ..., Error: _Optional[_Union[ServiceError, str]] = ..., ErrorMsg: _Optional[str] = ...) -> None: ...

class SetRequest(_message.Message):
    __slots__ = ("Header", "Pairs")
    HEADER_FIELD_NUMBER: _ClassVar[int]
    PAIRS_FIELD_NUMBER: _ClassVar[int]
    Header: Header
    Pairs: _containers.RepeatedCompositeFieldContainer[SetPair]
    def __init__(self, Header: _Optional[_Union[Header, _Mapping]] = ..., Pairs: _Optional[_Iterable[_Union[SetPair, _Mapping]]] = ...) -> None: ...

class SetResponse(_message.Message):
    __slots__ = ("Header", "Pairs", "Error", "ErrorMsg")
    HEADER_FIELD_NUMBER: _ClassVar[int]
    PAIRS_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    ERRORMSG_FIELD_NUMBER: _ClassVar[int]
    Header: Header
    Pairs: _containers.RepeatedCompositeFieldContainer[SetPair]
    Error: ServiceError
    ErrorMsg: str
    def __init__(self, Header: _Optional[_Union[Header, _Mapping]] = ..., Pairs: _Optional[_Iterable[_Union[SetPair, _Mapping]]] = ..., Error: _Optional[_Union[ServiceError, str]] = ..., ErrorMsg: _Optional[str] = ...) -> None: ...

class PointQueryRequest(_message.Message):
    __slots__ = ("Header", "Query", "Device", "Names", "Types", "Locations", "ConsiderDeviceLoc", "Resource", "Error", "ErrorMsg")
    HEADER_FIELD_NUMBER: _ClassVar[int]
    QUERY_FIELD_NUMBER: _ClassVar[int]
    DEVICE_FIELD_NUMBER: _ClassVar[int]
    NAMES_FIELD_NUMBER: _ClassVar[int]
    TYPES_FIELD_NUMBER: _ClassVar[int]
    LOCATIONS_FIELD_NUMBER: _ClassVar[int]
    CONSIDERDEVICELOC_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    ERRORMSG_FIELD_NUMBER: _ClassVar[int]
    Header: Header
    Query: str
    Device: str
    Names: _containers.RepeatedScalarFieldContainer[str]
    Types: _containers.RepeatedScalarFieldContainer[str]
    Locations: _containers.RepeatedScalarFieldContainer[str]
    ConsiderDeviceLoc: bool
    Resource: Dtype
    Error: QueryError
    ErrorMsg: str
    def __init__(self, Header: _Optional[_Union[Header, _Mapping]] = ..., Query: _Optional[str] = ..., Device: _Optional[str] = ..., Names: _Optional[_Iterable[str]] = ..., Types: _Optional[_Iterable[str]] = ..., Locations: _Optional[_Iterable[str]] = ..., ConsiderDeviceLoc: bool = ..., Resource: _Optional[_Union[Dtype, str]] = ..., Error: _Optional[_Union[QueryError, str]] = ..., ErrorMsg: _Optional[str] = ...) -> None: ...

class QueryResponse(_message.Message):
    __slots__ = ("Header", "Query", "Values", "Dtype", "Error", "ErrorMsg")
    HEADER_FIELD_NUMBER: _ClassVar[int]
    QUERY_FIELD_NUMBER: _ClassVar[int]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    DTYPE_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    ERRORMSG_FIELD_NUMBER: _ClassVar[int]
    Header: Header
    Query: str
    Values: _containers.RepeatedScalarFieldContainer[str]
    Dtype: Dtype
    Error: QueryError
    ErrorMsg: str
    def __init__(self, Header: _Optional[_Union[Header, _Mapping]] = ..., Query: _Optional[str] = ..., Values: _Optional[_Iterable[str]] = ..., Dtype: _Optional[_Union[Dtype, str]] = ..., Error: _Optional[_Union[QueryError, str]] = ..., ErrorMsg: _Optional[str] = ...) -> None: ...
